<?php

namespace Drupal\fluent;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldType\EntityReferenceItem;
use Drupal\field\Entity\FieldConfig;
use Drupal\fluent\Plugin\FluentFieldResolverManager;

class FieldResolver {

  private array $fieldResolvers = [];

  private $entity;

  public function __construct(FluentFieldResolverManager $plugin) {
    $this->pluginManager = $plugin;

    foreach ($this->pluginManager->getDefinitions() as $id => $definition) {
      $this->fieldResolvers[$id] = $this->pluginManager->createInstance($id);
    }

    $this->commonResolver = $this->fieldResolvers['common_field'];
    unset($this->fieldResolvers['common_field']);
  }

  public function resolve($entity, array $path, int $position) {
    $this->entity = $entity;

    // Save the current entity.
    $tmpEntity = $this->entity;
    $items = count($path);
    $count = 0;

    do {
      $possibleField = $path[$count];

      $solvedField = $this->getResolver($tmpEntity, $possibleField, $position);

      if ($solvedField === NULL) {
        throw new \Exception('Field not found');
      }

      $tmpEntity = $solvedField;

      if (is_array($solvedField)) {
        break;
      }

      if ($this->skipCount($solvedField)) {
        continue;
      }

      $count++;
    } while ($count < $items);

    return $tmpEntity;
  }


  private function skipCount($solvedField): bool {
    return (
      $solvedField instanceof FieldItemListInterface
      ||
      $solvedField instanceof EntityReferenceItem
    );
  }

  private function getResolver($entity, $name, $position = 0) {
    if (method_exists($entity, 'getFieldDefinition')) {
      /** @var FieldConfig $fieldDefinition */
      $fieldDefinition = $entity->getFieldDefinition($name);
      if ($fieldDefinition) {
        foreach ($this->fieldResolvers as $resolver) {
          if ($resolver->can($fieldDefinition)) {
            $cardinality = $fieldDefinition->getFieldStorageDefinition()->getCardinality();

            return $resolver->handler($entity, $name, [
              'position' => $position,
              'field_cardinality' => $cardinality,
            ]);
          }
        }
      }

      return $this->commonResolver->handler($entity, $name);
    } else {
      // TODO: don't fieldable entity
    }
  }
}
