<?php

namespace Drupal\fluent;

use Drupal\Core\Entity\EntityBase;
use Drupal\Core\Field\FieldItemInterface;

/**
 *
 */
class Fluent {

  /**
   * @var mixed
   */
  private $entity;

  /**
   * @var \Drupal\fluent\FieldItemResolver
   */
  private FieldItemResolver $fieldItemResolver;

  /**
   * @var \Drupal\fluent\FieldResolver
   */
  private FieldResolver $fieldResolver;

  /**
   *
   */
  public function __construct(FieldResolver $fieldResolver, FieldItemResolver $fieldItemResolver) {
    $this->fieldResolver = $fieldResolver;
    $this->fieldItemResolver = $fieldItemResolver;
  }

  /**
   * @param $entity
   * @return $this
   */
  public function using($entity) {
    $this->entity = $entity;

    return $this;
  }

  public function only(array $paths) {

  }

  /**
   * @param string $path
   * @param $defaultValue
   * @return mixed
   *
   * @throws \Exception
   */
  public function firstValue(string $path, $defaultValue = NULL) {
    return $this->nthValue($path, 0, $defaultValue)->first();
  }

  /**
   * @param string $path
   * @param array $default
   *
   * @return \Drupal\fluent\Collection
   * @throws \Exception
   */
  public function values(string $path, array $default = []): Collection {
    return $this->nthValue($path, -1, $default);
  }

  /**
   * @param string $path
   * @param int $position
   * @param $defaultValue
   *
   * @return \Drupal\fluent\Collection
   * @throws \Exception
   */
  public function nthValue(string $path, int $position, $defaultValue = NULL): Collection {
    $tmpEntity = $this->fieldResolver->resolve($this->entity, $this->getPath($path), $position);
    $field = $this->fieldItemResolver->resolve($tmpEntity, $this->getPath($path));

    if ($field instanceof EntityBase) {
      $collection = [$field];
    } else {
      $collection = $field;
    }

    return Collection::make($collection);
  }

  private function getPath(string $path): array {
    return explode('.', $path);
  }
}
